//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "BoltCalc.h"
#include "StringHelp.h"
#include <math.h>
#include <shellapi.h>
#include <clipbrd.hpp>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TBoltCalcForm *BoltCalcForm;
//---------------------------------------------------------------------------
__fastcall TBoltCalcForm::TBoltCalcForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TBoltCalcForm::FormCreate(TObject *Sender)
{
    //reading bolt cross sectional area
    AnsiString BoltDataFilename = ExtractFileDir(Application->ExeName) + "\\BoltData.csv";
    TStringList *inputfile = new TStringList();
    inputfile->LoadFromFile(BoltDataFilename);
    //inputfile->Delete(0);
    int counter = 0;
    AnsiString temp;
    //delete comments
    while (counter < inputfile->Count) {
        temp = inputfile->Strings[counter];
        if (LeftString(temp,1) == "#") inputfile->Delete(counter);
        counter++;
        }
    counter = 0;

    //interpret XSect area data
    BoltSize.Length = inputfile->Count;
    while (counter < inputfile->Count) {
        temp = inputfile->Strings[counter];
        BoltSize[counter].Name = ExtractWord(temp,",");
        if ( BoltSize[counter].Name == "") BoltSize[counter].Name = "M10";
        temp = DeleteWord(temp,",");
        BoltSize[counter].Area = ToDoubleDef(temp,49.49);
        counter++;
        }

    inputfile->Clear();

    //fill bolt size combo box
    for (int i = 0; i < BoltSize.Length; i++) {
        BoltSizeComboBox->Items->Add(BoltSize[i].Name);
        }

    if (BoltSizeComboBox->Items->Count > 4) {
        BoltSizeComboBox->ItemIndex = 4;
        } else {
        BoltSizeComboBox->ItemIndex = 0;
        }

    //reading bolt material data
    AnsiString BoltMaterialFilename = ExtractFileDir(Application->ExeName) + "\\BoltMaterial.csv";
    inputfile->LoadFromFile(BoltMaterialFilename);
    //inputfile->Delete(0);
    counter = 0;
    //delete comments
    while (counter < inputfile->Count) {
        temp = inputfile->Strings[counter];
        if (LeftString(temp,1) == "#") inputfile->Delete(counter);
        counter++;
        }
    counter = 0;

    //interpret XSect area data
    BoltMaterial.Length = inputfile->Count;
    while (counter < inputfile->Count) {
        temp = inputfile->Strings[counter];
        BoltMaterial[counter].Name = ExtractWord(temp,",");
        if ( BoltMaterial[counter].Name == "") BoltMaterial[counter].Name = "Grade 4.6";
        temp = DeleteWord(temp,",");
        BoltMaterial[counter].fty = ToDoubleDef(ExtractWord(temp,","),260);
        temp = DeleteWord(temp,",");
        BoltMaterial[counter].ftu = ToDoubleDef(temp,400);
        counter++;
        }

    delete inputfile;

    //fill bolt size combo box
    for (int i = 0; i < BoltMaterial.Length; i++) {
        BoltMaterialComboBox->Items->Add(BoltMaterial[i].Name);
        }

    BoltMaterialComboBox->ItemIndex = 0;

    //delete inputfile;
    Calculate(this);

}
//---------------------------------------------------------------------------

void __fastcall TBoltCalcForm::FormDestroy(TObject *Sender)
{
    BoltSize.Length = 0;
    BoltMaterial.Length = 0;
}
//---------------------------------------------------------------------------

void __fastcall TBoltCalcForm::Calculate(TObject *Sender)
{
    //Prevention of errors
    if (BoltSizeComboBox->ItemIndex == -1) BoltSizeComboBox->ItemIndex = 0;
    if (BoltMaterialComboBox->ItemIndex == -1) BoltMaterialComboBox->ItemIndex = 0;

    //Calculate Stresses.
    double axialload = ToDoubleDef(AxialLoadEditBox->Text,0.0);
    double shearload = ToDoubleDef(ShearLoadEditBox->Text,0.0);
    double axialstress = axialload/BoltSize[BoltSizeComboBox->ItemIndex].Area;
    double shearstress = shearload/BoltSize[BoltSizeComboBox->ItemIndex].Area;
    double p1stress = axialstress/2.0 + sqrt((axialstress/2.0)*(axialstress/2.0) + shearstress * shearstress);
    double p2stress = axialstress/2.0 - sqrt((axialstress/2.0)*(axialstress/2.0) + shearstress * shearstress);
    double vmstress = sqrt(p1stress * p1stress + p2stress * p2stress - p1stress * p2stress);
    double accelerationfactor = ToDoubleDef(AccelerationFactorEditBox->Text,1.5);
    double prooffactor = ToDoubleDef(ProofFactorEditBox->Text,2.0);
    double ultimatefactor = ToDoubleDef(UltimateFactorEditBox->Text,3.0);
    double RFy;
    if (vmstress == 0.0 || accelerationfactor == 0.0 || prooffactor == 0.0) {
        RFy = 1e6;
        } else {
        RFy = BoltMaterial[BoltMaterialComboBox->ItemIndex].fty/(vmstress * accelerationfactor * prooffactor);
        }
    double RFu;
    if (vmstress == 0.0 || accelerationfactor == 0.0 || ultimatefactor == 0.0) {
        RFu = 1e6;
        } else {
        RFu = BoltMaterial[BoltMaterialComboBox->ItemIndex].ftu/(vmstress * accelerationfactor * ultimatefactor);
        }

    //Display Stresses
    AxialStressEditBox->Text = DP(2,axialstress);
    ShearStressEditBox->Text = DP(2,shearstress);
    P1StressEditBox->Text = DP(2,p1stress);
    P2StressEditBox->Text = DP(2,p2stress);
    VMStressEditBox->Text = DP(2,vmstress);
    RFyEditBox->Text = DP(2,RFy);
    RFuEditBox->Text = DP(2,RFu);
    if (RFy >= 1.00) {
        RFyEditBox->Color = clBtnFace;
        } else {
        RFyEditBox->Color = clRed;
        }
    if (RFu >= 1.00) {
        RFuEditBox->Color = clBtnFace;
        } else {
        RFuEditBox->Color = clRed;
        }
    BoltXSectEditBox->Text = DP(2,BoltSize[BoltSizeComboBox->ItemIndex].Area);
    ProofStressEditBox->Text = DP(2,BoltMaterial[BoltMaterialComboBox->ItemIndex].fty);
    UltimateStressEditBox->Text = DP(2,BoltMaterial[BoltMaterialComboBox->ItemIndex].ftu);

}
//---------------------------------------------------------------------------

void __fastcall TBoltCalcForm::HelpButtonClick(TObject *Sender)
{
	AnsiString Location = ExtractFileDir(Application->ExeName) + "\\BoltStressesHelp.pdf";
	AnsiString action = "open";
	ShellExecute(Application->Handle,action.c_str(),Location.c_str(),Location.c_str(),NULL,SW_SHOW);

}
//---------------------------------------------------------------------------
void __fastcall TBoltCalcForm::CopyButtonClick(TObject *Sender)
{
    HelpButton->Visible = false;
    CopyButton->Visible = false;
    Refresh();
    Graphics::TBitmap * Bitmap = new Graphics::TBitmap();
    Bitmap->Width = ClientWidth;
    Bitmap->Height = ClientHeight;
    Bitmap->Canvas->CopyRect(ClientRect,Canvas,ClientRect);
    unsigned int DataHandle = 0;
    HPALETTE APalette;
    unsigned short MyFormat = CF_BITMAP;
    try {
        try {
            Bitmap->SaveToClipboardFormat(MyFormat,DataHandle,APalette);
            Clipboard()->SetAsHandle(CF_BITMAP,DataHandle);
            }

        catch (...) {
            ShowMessage("Sorry, Failed to copy image to clipboard.");
            }
        }

    __finally {
        HelpButton->Visible = true;
        CopyButton->Visible = true;
        }

    delete Bitmap;

}
//---------------------------------------------------------------------------

